@@ -7,34 +7,34 @@ from concurrent.futures import ThreadPoolExecutor
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from unittest import skipIf, TestCase
-from unittest.mock import call, patch
+from unittest.mock import call, Mock, patch
 
 import trailrunner
 from click.testing import CliRunner
 from libcst import ParserSyntaxError
 
 from ufmt.cli import echo_results, main
-from ufmt.core import Result
+from ufmt.types import Result
 
 from .core import CORRECTLY_FORMATTED_CODE, POORLY_FORMATTED_CODE
 
 
 @patch.object(trailrunner.core.Trailrunner, "DEFAULT_EXECUTOR", ThreadPoolExecutor)
 class CliTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.runner = CliRunner(mix_stderr=False)
         self.cwd = os.getcwd()
         self.td = TemporaryDirectory()
         self.tdp = Path(self.td.name).resolve()
         os.chdir(self.tdp)
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         os.chdir(self.cwd)
         self.td.cleanup()
 
     @patch("ufmt.cli.echo_color_precomputed_diff")
     @patch("ufmt.cli.click.secho")
-    def test_echo(self, echo_mock, mol_mock):
+    def test_echo(self, echo_mock: Mock, mol_mock: Mock) -> None:
         f1 = Path("foo/bar.py")
         f2 = Path("fuzz/buzz.py")
         f3 = Path("make/rake.py")
@@ -100,14 +100,16 @@ class CliTest(TestCase):
             mol_mock.reset_mock()
 
     @patch("ufmt.cli.ufmt_paths")
-    def test_check(self, ufmt_mock):
+    def test_check(self, ufmt_mock: Mock) -> None:
         with self.subTest("no paths given"):
             ufmt_mock.reset_mock()
             ufmt_mock.return_value = []
             result = self.runner.invoke(main, ["check"])
-            ufmt_mock.assert_called_with([Path(".")], dry_run=True)
+            ufmt_mock.assert_called_with(
+                [Path(".")], dry_run=True, concurrency=None, root=None
+            )
             self.assertRegex(result.stderr, r"No files found")
-            self.assertEqual(1, result.exit_code)
+            self.assertEqual(0, result.exit_code)
 
         with self.subTest("already formatted"):
             ufmt_mock.reset_mock()
@@ -117,7 +119,10 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["check", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                concurrency=None,
+                root=None,
             )
             self.assertEqual(0, result.exit_code)
 
@@ -129,7 +134,10 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["check", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                concurrency=None,
+                root=None,
             )
             self.assertEqual(1, result.exit_code)
 
@@ -149,7 +157,10 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["check", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                concurrency=None,
+                root=None,
             )
             self.assertRegex(
                 result.stderr, r"Error formatting .*frob\.py: Syntax Error @ 4:16"
@@ -162,19 +173,23 @@ class CliTest(TestCase):
                 Result(Path("foo.py"), skipped="special"),
             ]
             result = self.runner.invoke(main, ["check", "foo.py"])
-            ufmt_mock.assert_called_with([Path("foo.py")], dry_run=True)
+            ufmt_mock.assert_called_with(
+                [Path("foo.py")], dry_run=True, concurrency=None, root=None
+            )
             self.assertRegex(result.stderr, r"Skipped .*foo\.py: special")
             self.assertEqual(0, result.exit_code)
 
     @patch("ufmt.cli.ufmt_paths")
-    def test_diff(self, ufmt_mock):
+    def test_diff(self, ufmt_mock: Mock) -> None:
         with self.subTest("no paths given"):
             ufmt_mock.reset_mock()
             ufmt_mock.return_value = []
             result = self.runner.invoke(main, ["diff"])
-            ufmt_mock.assert_called_with([Path(".")], dry_run=True, diff=True)
+            ufmt_mock.assert_called_with(
+                [Path(".")], dry_run=True, diff=True, concurrency=None, root=None
+            )
             self.assertRegex(result.stderr, r"No files found")
-            self.assertEqual(1, result.exit_code)
+            self.assertEqual(0, result.exit_code)
 
         with self.subTest("already formatted"):
             ufmt_mock.reset_mock()
@@ -184,7 +199,11 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["diff", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True, diff=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                diff=True,
+                concurrency=None,
+                root=None,
             )
             self.assertEqual(0, result.exit_code)
 
@@ -196,7 +215,11 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["diff", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True, diff=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                diff=True,
+                concurrency=None,
+                root=None,
             )
             self.assertEqual(1, result.exit_code)
 
@@ -216,7 +239,11 @@ class CliTest(TestCase):
             ]
             result = self.runner.invoke(main, ["diff", "bar.py", "foo/frob.py"])
             ufmt_mock.assert_called_with(
-                [Path("bar.py"), Path("foo/frob.py")], dry_run=True, diff=True
+                [Path("bar.py"), Path("foo/frob.py")],
+                dry_run=True,
+                diff=True,
+                concurrency=None,
+                root=None,
             )
             self.assertRegex(
                 result.stderr, r"Error formatting .*frob\.py: Syntax Error @ 4:16"
@@ -229,7 +256,9 @@ class CliTest(TestCase):
                 Result(Path("foo.py"), skipped="special"),
             ]
             result = self.runner.invoke(main, ["diff", "foo.py"])
-            ufmt_mock.assert_called_with([Path("foo.py")], dry_run=True, diff=True)
+            ufmt_mock.assert_called_with(
+                [Path("foo.py")], dry_run=True, diff=True, concurrency=None, root=None
+            )
             self.assertRegex(result.stderr, r"Skipped .*foo\.py: special")
             self.assertEqual(0, result.exit_code)
 
@@ -239,19 +268,35 @@ class CliTest(TestCase):
                 Result(Path("foo.py"), skipped="special"),
             ]
             result = self.runner.invoke(main, ["--quiet", "diff", "foo.py"])
-            ufmt_mock.assert_called_with([Path("foo.py")], dry_run=True, diff=True)
+            ufmt_mock.assert_called_with(
+                [Path("foo.py")], dry_run=True, diff=True, concurrency=None, root=None
+            )
             self.assertEqual("", result.stderr)
             self.assertEqual(0, result.exit_code)
 
+        with self.subTest("bad root dir"):
+            ufmt_mock.reset_mock()
+            ufmt_mock.return_value = [
+                Result(Path("bar.py"), changed=False),
+            ]
+            result = self.runner.invoke(
+                main, ["--root", "DOES_NOT_EXIST", "diff", "bar.py"]
+            )
+            self.assertIn(
+                r"Directory 'DOES_NOT_EXIST' does not exist",
+                result.stderr,
+            )
+            self.assertEqual(2, result.exit_code)
+
     @patch("ufmt.cli.ufmt_paths")
-    def test_format(self, ufmt_mock):
+    def test_format(self, ufmt_mock: Mock) -> None:
         with self.subTest("no paths given"):
             ufmt_mock.reset_mock()
             ufmt_mock.return_value = []
             result = self.runner.invoke(main, ["format"])
-            ufmt_mock.assert_called_with([Path(".")])
+            ufmt_mock.assert_called_with([Path(".")], concurrency=None, root=None)
             self.assertRegex(result.stderr, r"No files found")
-            self.assertEqual(1, result.exit_code)
+            self.assertEqual(0, result.exit_code)
 
         with self.subTest("already formatted"):
             ufmt_mock.reset_mock()
@@ -260,7 +305,9 @@ class CliTest(TestCase):
                 Result(Path("foo/frob.py"), changed=False),
             ]
             result = self.runner.invoke(main, ["format", "bar.py", "foo/frob.py"])
-            ufmt_mock.assert_called_with([Path("bar.py"), Path("foo/frob.py")])
+            ufmt_mock.assert_called_with(
+                [Path("bar.py"), Path("foo/frob.py")], concurrency=None, root=None
+            )
             self.assertEqual(0, result.exit_code)
 
         with self.subTest("needs formatting"):
@@ -270,7 +317,9 @@ class CliTest(TestCase):
                 Result(Path("foo/frob.py"), changed=True),
             ]
             result = self.runner.invoke(main, ["format", "bar.py", "foo/frob.py"])
-            ufmt_mock.assert_called_with([Path("bar.py"), Path("foo/frob.py")])
+            ufmt_mock.assert_called_with(
+                [Path("bar.py"), Path("foo/frob.py")], concurrency=None, root=None
+            )
             self.assertEqual(0, result.exit_code)
 
         with self.subTest("syntax error"):
@@ -288,10 +337,13 @@ class CliTest(TestCase):
                 ),
             ]
             result = self.runner.invoke(main, ["format", "bar.py", "foo/frob.py"])
-            ufmt_mock.assert_called_with([Path("bar.py"), Path("foo/frob.py")])
+            ufmt_mock.assert_called_with(
+                [Path("bar.py"), Path("foo/frob.py")], concurrency=None, root=None
+            )
             self.assertRegex(
                 result.stderr, r"Error formatting .*frob\.py: Syntax Error @ 4:16"
             )
+            self.assertRegex(result.stderr, " 1 error, 1 file already formatted")
             self.assertEqual(1, result.exit_code)
 
         with self.subTest("skipped file"):
@@ -300,7 +352,7 @@ class CliTest(TestCase):
                 Result(Path("foo.py"), skipped="special"),
             ]
             result = self.runner.invoke(main, ["format", "foo.py"])
-            ufmt_mock.assert_called_with([Path("foo.py")])
+            ufmt_mock.assert_called_with([Path("foo.py")], concurrency=None, root=None)
             self.assertRegex(result.stderr, r"Skipped .*foo\.py: special")
             self.assertEqual(0, result.exit_code)
 
@@ -366,14 +418,14 @@ class CliTest(TestCase):
             self.assertIn("Formatted hello.py\n", result.stderr)
             self.assertEqual(0, result.exit_code)
 
-    def test_end_to_end(self):
+    def test_end_to_end(self) -> None:
         alpha = self.tdp / "alpha.py"
         beta = self.tdp / "beta.py"
         (self.tdp / "sub").mkdir()
         gamma = self.tdp / "sub" / "gamma.py"
         kappa = self.tdp / "sub" / "kappa.py"
 
-        def reset():
+        def reset() -> None:
             alpha.write_text(CORRECTLY_FORMATTED_CODE)
             beta.write_text(POORLY_FORMATTED_CODE)
             gamma.write_text(CORRECTLY_FORMATTED_CODE)
