@@ -1,14 +1,18 @@
-# Copyright 2021 John Reese
+# Copyright 2022 Amethyst Reese
 # Licensed under the MIT license
 
 import io
+import warnings
 from concurrent.futures import ThreadPoolExecutor
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from unittest import TestCase
 from unittest.mock import call, Mock, patch
 
+import black
+import ruff_api
 import trailrunner
+import usort
 from libcst import ParserSyntaxError
 
 import ufmt
@@ -17,9 +21,12 @@ from ufmt.types import (
     BlackConfig,
     Encoding,
     FileContent,
+    Formatter,
     Result,
     SkipFormatting,
+    Sorter,
     STDIN,
+    UfmtConfig,
     UsortConfig,
 )
 
@@ -110,11 +117,14 @@ print "hello world"
 class CoreTest(TestCase):
     maxDiff = None
 
-    def test_ufmt_bytes(self):
+    @patch("ufmt.core.black.format_file_contents", wraps=black.format_file_contents)
+    @patch("ufmt.core.ruff_api.format_string", wraps=ruff_api.format_string)
+    def test_ufmt_bytes(self, ruff_mock: Mock, black_mock: Mock) -> None:
         black_config = BlackConfig()
         usort_config = UsortConfig()
 
         with self.subTest("changed"):
+            black_mock.reset_mock()
             result = ufmt.ufmt_bytes(
                 Path("foo.py"),
                 POORLY_FORMATTED_CODE.encode(),
@@ -122,8 +132,11 @@ class CoreTest(TestCase):
                 usort_config=usort_config,
             )
             self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            black_mock.assert_called_once()
+            ruff_mock.assert_not_called()
 
         with self.subTest("unchanged"):
+            black_mock.reset_mock()
             result = ufmt.ufmt_bytes(
                 Path("foo.py"),
                 CORRECTLY_FORMATTED_CODE.encode(),
@@ -131,8 +144,11 @@ class CoreTest(TestCase):
                 usort_config=usort_config,
             )
             self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            black_mock.assert_called_once()
+            ruff_mock.assert_not_called()
 
         with self.subTest("type stub"):
+            black_mock.reset_mock()
             result = ufmt.ufmt_bytes(
                 Path("foo.pyi"),
                 POORLY_FORMATTED_STUB.encode(),
@@ -140,8 +156,142 @@ class CoreTest(TestCase):
                 usort_config=usort_config,
             )
             self.assertEqual(CORRECTLY_FORMATTED_STUB.encode(), result)
+            black_mock.assert_called_once()
+            ruff_mock.assert_not_called()
 
-    def test_ufmt_bytes_pre_processor(self):
+    @patch("ufmt.core.black.format_file_contents", wraps=black.format_file_contents)
+    @patch("ufmt.core.ruff_api.format_string", wraps=ruff_api.format_string)
+    def test_ufmt_bytes_alternate_formatter(
+        self, ruff_mock: Mock, black_mock: Mock
+    ) -> None:
+        black_config = BlackConfig()
+        usort_config = UsortConfig()
+
+        with self.subTest("changed"):
+            ruff_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                POORLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(formatter=Formatter.ruff_api),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            ruff_mock.assert_called_once()
+            black_mock.assert_not_called()
+
+        with self.subTest("unchanged"):
+            ruff_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                CORRECTLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(formatter=Formatter.ruff_api),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            ruff_mock.assert_called_once()
+            black_mock.assert_not_called()
+
+        with self.subTest("type stub"):
+            ruff_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.pyi"),
+                POORLY_FORMATTED_STUB.encode(),
+                ufmt_config=UfmtConfig(formatter=Formatter.ruff_api),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_STUB.encode(), result)
+            ruff_mock.assert_called_once()
+            black_mock.assert_not_called()
+
+        with self.subTest("unsupported formatter"):
+            with self.assertRaisesRegex(
+                ValueError, "'garbage' is not a supported formatter"
+            ):
+                ufmt.ufmt_bytes(
+                    Path("foo.pyi"),
+                    POORLY_FORMATTED_STUB.encode(),
+                    ufmt_config=UfmtConfig(formatter="garbage"),  # type:ignore
+                    black_config=black_config,
+                    usort_config=usort_config,
+                )
+            ruff_mock.assert_called_once()
+            black_mock.assert_not_called()
+
+    @patch("ufmt.core.usort", wraps=usort.usort)
+    @patch("ufmt.core.ruff_api.isort_string", wraps=ruff_api.isort_string)
+    def test_ufmt_bytes_alternate_sorter(
+        self, ruff_mock: Mock, usort_mock: Mock
+    ) -> None:
+        black_config = BlackConfig()
+        usort_config = UsortConfig()
+
+        with self.subTest("default"):
+            usort_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                POORLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            usort_mock.assert_called_once()
+
+        with self.subTest("usort"):
+            usort_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                POORLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            usort_mock.assert_called_once()
+
+        with self.subTest("ruff-api"):
+            usort_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                POORLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(sorter=Sorter.ruff_api),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), result)
+            usort_mock.assert_not_called()
+            ruff_mock.assert_called_once()
+
+        with self.subTest("skip"):
+            usort_mock.reset_mock()
+            result = ufmt.ufmt_bytes(
+                Path("foo.py"),
+                POORLY_FORMATTED_CODE.encode(),
+                ufmt_config=UfmtConfig(sorter=Sorter.skip),
+                black_config=black_config,
+                usort_config=usort_config,
+            )
+            expected = POORLY_FORMATTED_CODE[:64] + CORRECTLY_FORMATTED_CODE[65:]
+            self.assertEqual(expected.encode(), result)
+            usort_mock.assert_not_called()
+
+        with self.subTest("unsupported sorter"):
+            with self.assertRaisesRegex(
+                ValueError, "'garbage' is not a supported sorter"
+            ):
+                ufmt.ufmt_bytes(
+                    Path("foo.py"),
+                    POORLY_FORMATTED_CODE.encode(),
+                    ufmt_config=UfmtConfig(sorter="garbage"),  # type:ignore
+                    black_config=black_config,
+                    usort_config=usort_config,
+                )
+            usort_mock.assert_not_called()
+
+    def test_ufmt_bytes_pre_processor(self) -> None:
         def pre_processor(
             path: Path, content: bytes, *, encoding: Encoding = "utf-8"
         ) -> bytes:
@@ -162,7 +312,7 @@ class CoreTest(TestCase):
             CORRECTLY_FORMATTED_CODE.encode() + b'\n\nprint("hello")\n', result
         )
 
-    def test_ufmt_bytes_post_processor(self):
+    def test_ufmt_bytes_post_processor(self) -> None:
         def post_processor(
             path: Path, content: bytes, *, encoding: Encoding = "utf-8"
         ) -> bytes:
@@ -183,10 +333,12 @@ class CoreTest(TestCase):
             CORRECTLY_FORMATTED_CODE.encode() + b"\nprint('hello')\n", result
         )
 
-    def test_ufmt_string(self):
+    def test_ufmt_string(self) -> None:
         black_config = BlackConfig()
         usort_config = UsortConfig()
 
+        warnings.simplefilter("ignore", DeprecationWarning)
+
         with self.subTest("changed"):
             result = ufmt.ufmt_string(
                 Path("foo.py"),
@@ -214,10 +366,15 @@ class CoreTest(TestCase):
             )
             self.assertEqual(CORRECTLY_FORMATTED_STUB, result)
 
-    def test_ufmt_file(self):
+        warnings.resetwarnings()
+
+        with self.subTest("version check"):
+            self.assertRegex(ufmt.__version__, r"^2\.", "remove ufmt_string in 3.0")
+
+    def test_ufmt_file(self) -> None:
         with TemporaryDirectory() as td:
-            td = Path(td)
-            f = td / "foo.py"
+            tdp = Path(td)
+            f = tdp / "foo.py"
             f.write_text(POORLY_FORMATTED_CODE)
 
             with self.subTest("dry run"):
@@ -313,7 +470,7 @@ class CoreTest(TestCase):
 
     @patch("ufmt.core.sys.stdin")
     @patch("ufmt.core.sys.stdout")
-    def test_ufmt_stdin(self, stdout_mock, stdin_mock):
+    def test_ufmt_stdin(self, stdout_mock: Mock, stdin_mock: Mock) -> None:
         with self.subTest("check"):
             stdin_mock.buffer = stdin = io.BytesIO()
             stdout_mock.buffer = stdout = io.BytesIO()
@@ -350,7 +507,7 @@ class CoreTest(TestCase):
             result = ufmt_stdin(path, dry_run=True, diff=True)
             self.assertIsNotNone(result.diff)
             self.assertRegex(
-                result.diff, r"--- hello.world\.py\n\+\+\+ hello.world\.py"
+                result.diff or "", r"--- hello.world\.py\n\+\+\+ hello.world\.py"
             )
             stdout.seek(0)
             self.assertEqual(b"", stdout.read())
@@ -367,11 +524,11 @@ class CoreTest(TestCase):
             stdout.seek(0)
             self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), stdout.read())
 
-    def test_ufmt_paths(self):
+    def test_ufmt_paths(self) -> None:
         with TemporaryDirectory() as td:
-            td = Path(td)
-            f1 = td / "bar.py"
-            sd = td / "foo"
+            tdp = Path(td)
+            f1 = tdp / "bar.py"
+            sd = tdp / "foo"
             sd.mkdir()
             f2 = sd / "baz.py"
             f3 = sd / "frob.py"
@@ -389,7 +546,7 @@ class CoreTest(TestCase):
                 with self.subTest("non-existent paths"):
                     results = list(
                         ufmt.ufmt_paths(
-                            [(td / "fake.py"), (td / "another.py")], dry_run=True
+                            [(tdp / "fake.py"), (tdp / "another.py")], dry_run=True
                         )
                     )
                     self.assertEqual([], results)
@@ -410,6 +567,7 @@ class CoreTest(TestCase):
                                 dry_run=True,
                                 diff=False,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -420,6 +578,7 @@ class CoreTest(TestCase):
                                 dry_run=True,
                                 diff=False,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -441,6 +600,7 @@ class CoreTest(TestCase):
                                 dry_run=True,
                                 diff=True,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -451,6 +611,7 @@ class CoreTest(TestCase):
                                 dry_run=True,
                                 diff=True,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -471,6 +632,7 @@ class CoreTest(TestCase):
                                 dry_run=False,
                                 diff=False,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -481,6 +643,7 @@ class CoreTest(TestCase):
                                 dry_run=False,
                                 diff=False,
                                 return_content=False,
+                                ufmt_config_factory=None,
                                 black_config_factory=None,
                                 usort_config_factory=None,
                                 pre_processor=None,
@@ -493,7 +656,7 @@ class CoreTest(TestCase):
                     file_wrapper.reset_mock()
 
     @patch("ufmt.core.ufmt_stdin")
-    def test_ufmt_paths_stdin(self, stdin_mock):
+    def test_ufmt_paths_stdin(self, stdin_mock: Mock) -> None:
         stdin_mock.return_value = Result(path=STDIN, changed=True)
 
         with self.subTest("no name"):
@@ -503,6 +666,7 @@ class CoreTest(TestCase):
                 dry_run=True,
                 diff=False,
                 return_content=False,
+                ufmt_config_factory=None,
                 black_config_factory=None,
                 usort_config_factory=None,
                 pre_processor=None,
@@ -516,6 +680,7 @@ class CoreTest(TestCase):
                 dry_run=True,
                 diff=False,
                 return_content=False,
+                ufmt_config_factory=None,
                 black_config_factory=None,
                 usort_config_factory=None,
                 pre_processor=None,
@@ -532,13 +697,18 @@ class CoreTest(TestCase):
 
     @patch("ufmt.core.sys.stdin")
     @patch("ufmt.core.sys.stdout")
-    def test_ufmt_paths_stdin_resolves(self, stdout_mock, stdin_mock):
+    def test_ufmt_paths_stdin_resolves(
+        self, stdout_mock: Mock, stdin_mock: Mock
+    ) -> None:
         stdin_mock.buffer = io.BytesIO(POORLY_FORMATTED_CODE.encode())
         stdout_mock.buffer = stdout = io.BytesIO()
 
-        def fake_preprocessor(path, content, encoding):
+        def fake_preprocessor(
+            path: Path, content: FileContent, encoding: Encoding
+        ) -> FileContent:
             # ensure the fake path resolves correctly (#94)
             path.resolve()
+            return content
 
         result = list(ufmt.ufmt_paths([STDIN]))
         expected = [Result(Path("stdin"), changed=True, written=True)]
@@ -548,10 +718,10 @@ class CoreTest(TestCase):
         self.assertListEqual(expected, result)
         self.assertEqual(CORRECTLY_FORMATTED_CODE.encode(), output)
 
-    def test_ufmt_paths_config(self):
+    def test_ufmt_paths_config(self) -> None:
         with TemporaryDirectory() as td:
-            td = Path(td).resolve()
-            md = td / "foo"
+            tdp = Path(td).resolve()
+            md = tdp / "foo"
             md.mkdir()
             f1 = md / "__init__.py"
             f2 = md / "foo.py"
@@ -562,12 +732,12 @@ class CoreTest(TestCase):
             for f in f1, f2, f3:
                 f.write_text(POORLY_FORMATTED_CODE)
 
-            pyproj = td / "pyproject.toml"
+            pyproj = tdp / "pyproject.toml"
             pyproj.write_text(FAKE_CONFIG)
 
             file_wrapper = Mock(name="ufmt_file", wraps=ufmt.ufmt_file)
             with patch("ufmt.core.ufmt_file", file_wrapper):
-                list(ufmt.ufmt_paths([td]))
+                list(ufmt.ufmt_paths([tdp]))
                 file_wrapper.assert_has_calls(
                     [
                         call(
@@ -575,6 +745,7 @@ class CoreTest(TestCase):
                             dry_run=False,
                             diff=False,
                             return_content=False,
+                            ufmt_config_factory=None,
                             black_config_factory=None,
                             usort_config_factory=None,
                             pre_processor=None,
@@ -587,7 +758,7 @@ class CoreTest(TestCase):
                 self.assertEqual(f2.read_text(), CORRECTLY_FORMATTED_CODE)
                 self.assertEqual(f3.read_text(), POORLY_FORMATTED_CODE)
 
-    def test_e2e_empty_files(self):
+    def test_e2e_empty_files(self) -> None:
         with TemporaryDirectory() as td:
             tdp = Path(td).resolve()
             foo = tdp / "foo.py"
@@ -605,10 +776,10 @@ class CoreTest(TestCase):
             ]
             self.assertListEqual(expected, results)
 
-    def test_e2e_return_bytes(self):
+    def test_e2e_return_bytes(self) -> None:
         with TemporaryDirectory() as td:
-            td = Path(td).resolve()
-            foo = td / "foo.py"
+            tdp = Path(td).resolve()
+            foo = tdp / "foo.py"
 
             with self.subTest("unix newlines"):
                 foo.write_bytes(POORLY_FORMATTED_CODE.encode())
